@@ -12,7 +12,10 @@ such as Sun Grid Engine (SGE) or Condor.
 =head1 SYNOPSIS
 
  use Grid::Request;
- my $request = Grid::Request->new( project => "SomeProject" );
+ my $request = Grid::Request->new();
+
+ # Optionally set the job's project, if the configured scheduler requires it.
+ $request->project("MyProject");
 
  $request->times(2);
  $request->command("/path/to/executable");
@@ -41,8 +44,8 @@ such as Sun Grid Engine (SGE) or Condor.
  my $times_wrong_way = $request->get_times(3);
 
  # Finally, submit the request...
- my @id = $request->submit();
- print "The first ID for this request is $id[0].\n";
+ my @job_ids = $request->submit();
+ print "The first ID for this request is $job_ids[0].\n";
 
  # ...and wait for the results. This step is not necessary, only
  # if you wish to block, or wait for the request to complete before
@@ -61,6 +64,10 @@ Grid::Request->new(%args);
 B<Description:> This is the object constructor. Parameters are passed to
 the constructor in the form of a hash. Examples:
 
+  my $req = Grid::Request->new();
+
+  or
+
   my $req = Grid::Request->new( project => "SomeProject" );
 
   or
@@ -74,38 +81,33 @@ the constructor in the form of a hash. Examples:
 Users may also add a "debug" flag to the constructor call for increased
 reporting:
 
-  my $req = Grid::Request->new( project => "SomeProject",
-                                debug   => 1 );
+  my $req = Grid::Request->new( debug => 1 );
 
-B<Parameters:> Only the 'project' parameter is mandatory when calling the
-constructor.
+B<Parameters:> A hash of request configuration options.
 
 B<Returns:> $obj, a Grid::Request object.
 
 =head1 CONFIGURATION
 
-By default, the configuration file to that is used to determine what grid
-engine type to use and where to store temporary files is located in the
-invoking user's home directory under ~/.grid_request.conf. The file needs
-needs to have a [request] header and configurations for the 'tempdir' and
-'drm' parameters. The following is an example:
+By default, the configuration file that is used to determine what grid engine
+type to use and where to store temporary files is located in the invoking
+user's home directory under ~/.grid_request.conf. The file needs needs to have
+a [request] header and entries for the 'tempdir' and 'drm' parameters.  In addition,
+the file may also specify the path to a Log::Log4perl configuration file with the
+'log4perl-conf' entry name.The following is an example:
 
       [request]
-      tempdir=/path/to/grid/accessible/tmp/directory
       drm=SGE
+      tempdir=/path/to/grid/accessible/tmp/directory
+      log4perl-conf=/path/to/custom-log4perl.conf
 
 The 'tempdir' directory must point to a directory that is accessible
 to the grid execution machines, for instance, over NFS...
-Users may provide an alternate path to a different configuration file
-by specifying the 'config' parameter to the constructor:
 
-  my $req = Grid::Request->new( project => "SomeProject",
-                                config => "/some/other/dir/request.conf",
-                              );
 Another way of specifying an alternate configuration is to define
 the GRID_CONFIG environment variable.
 
-=head2 Class and object methods
+=head1 CLASS AND OBJECT METHODS
 
 =over 4
 
@@ -135,10 +137,8 @@ my $section = $Grid::Request::HTC::config_section;
 
 my $WORKER = $Grid::Request::HTC::WORKER;
 
-my $command_element = 0;
 my $DRMAA_INITIALIZED = 0;
-my $DEFAULT_BLOCK_SIZE = 100;
-our $VERSION = qw$Revision: 8365 $[1];
+our $VERSION = '0.11';
 my $SESSION_NAME = lc(__PACKAGE__);
 $SESSION_NAME =~ s/:+/_/g;
 
@@ -151,7 +151,7 @@ BEGIN: {
     require 5.006_00; # Make sure we're not running some old Perl.
 
     # Here we set up which methods go where.
-    my @command_meths = qw(account add_param class cmd_type command
+    my @command_meths = qw(account add_param block_size class cmd_type command
                            email end_time getenv project error hosts initialdir
                            input length memory name opsys output
                            priority start_time state times runtime evictable
@@ -163,6 +163,7 @@ BEGIN: {
 }
 
 
+# The constructor.
 sub new {
     my ($class, %args) = @_;
     my $self = bless {}, $class || ref($class);
@@ -181,7 +182,7 @@ sub new {
     # Here we separate our arguments to route them to the right class.
     my $command_args = $mapper->( sort keys %comm_meths );
 
-    # $config will hold the location of the configuration file for the,
+    # $config will hold the location of the configuration file for the
     # module, which holds the logger configuration, etc...
     # The user may specify it with a "config" parameter in the
     # constructor.
@@ -192,38 +193,7 @@ sub new {
     return $self;
 }
 
-# Private method only. Used to configure logging.
-sub _init_config_logger {
-    my $config = shift;
-    # TODO: Since Grid::HTC already parsed the default config file, use
-    # an accessor to get that config object rather than reparsing it
-    # here. This will involve adding an additional method in Grid::HTC.
-    my $default_cfg_obj = Config::IniFiles->new( -file => $default_config );
-    my ($cfg, $same_configs);
-    if (defined $config && ($config eq $default_config)) {
-        $same_configs = 1;
-        $cfg = $default_cfg_obj;
-    } else {
-        $cfg = Config::IniFiles->new(-file   => $config,
-                                     -import => $default_cfg_obj);
-    }
-
-    # Parse the location of the logger configuration and initialize
-    # Log4perl, if it has not already been initialized.
-    my $logger_conf = $cfg->val($section, "Log4perlConf");
-    Log::Log4perl->init_once($logger_conf);
-
-    # The currently installed version of Log::Log4perl (0.34) exhibits
-    # a problem when you check the return value of init. Per the perldoc
-    # documentation, you should be able to do an "or die..." on the init
-    # call, but it fails every time then. Check later releases, and if it's
-    # fixed, install and uncomment the next line.
-    # or croak "Could not initialize logging with $logger_conf.";
-    $logger = get_logger(__PACKAGE__);
-
-    return $cfg;
-}
-
+# Initialize the object. This is a private method. Do not call directly.
 sub _init {
     # Initialize the Request object. We need to parse the configuration
     # file, initialize the logger, create the Command object.
@@ -240,7 +210,7 @@ sub _init {
 
     $logger->info("Creating the first Command object.");
     # holds an array of command elements.
-    $self->{_cmd_ele}->[$command_element] =
+    $self->{_cmd_ele}->[0] =
         Grid::Request::Command->new(%$command_args_ref);
 
     $self->{_drm} = _load_drm($cfg);
@@ -252,10 +222,42 @@ sub _init {
     _init_drmaa();
 }
 
+# Private method only. Used to configure logging.
+sub _init_config_logger {
+    my $config = shift;
+    # TODO: Since Grid::Request::HTC already parsed the default config file, use
+    # an accessor to get that config object rather than reparsing it
+    # here. This will involve adding an additional method in Grid::Request::HTC.
+    my $default_cfg_obj = Config::IniFiles->new( -file => $default_config );
+    my ($cfg, $same_configs);
+    if (defined $config && ($config eq $default_config)) {
+        $same_configs = 1;
+        $cfg = $default_cfg_obj;
+    } else {
+        $cfg = Config::IniFiles->new(-file   => $config,
+                                     -import => $default_cfg_obj);
+    }
+
+    # Parse the location of the logger configuration and initialize
+    # Log4perl, if it has not already been initialized.
+    my $logger_conf = $cfg->val($section, "log4perl-conf");
+    if (defined $logger_conf) {
+        if (-f $logger_conf && -r $logger_conf) {
+            Log::Log4perl->init_once($logger_conf);
+        } else {
+            warn "Unable to configure logging with $logger_conf.\n";
+        }
+    }
+
+    $logger = get_logger(__PACKAGE__);
+
+    return $cfg;
+}
+
 # Accessors (private) used to return internal objects.
 sub _drm { return $_[0]->{_drm}; }
-sub _com_obj_list { return $_[0]->{_cmd_ele}; }
-sub _com_obj { return $_[0]->{_cmd_ele}->[$command_element]; }
+sub _com_obj_list { return wantarray ? @{ $_[0]->{_cmd_ele} } : $_[0]->{_cmd_ele}; }
+sub _com_obj { return $_[0]->{_cmd_ele}->[-1]; }
 sub _config { return $_[0]->{_config}; }
 
 sub _load_drm {
@@ -314,12 +316,12 @@ sub _validate {
     my $self = shift;
     $logger->debug("In _validate.");
     my $rv = 1;
-    if ($self->project() =~ m/\s/) {
-        Grid::Request::Exception->throw("White space is not allowed for the project attribute.");
+    if ($self->project() && ($self->project() =~ m/\s/)) {
+        Grid::Request::Exception->throw("White space is not allowed for the 'project' attribute.");
     }
 
-    if ($self->account() =~ m/\s/) {
-        Grid::Request::Exception->throw("White space is not allowed for the account attribute.");
+    if (defined($self->account()) && length($self->account()) && $self->account() =~ m/\s/) {
+        Grid::Request::Exception->throw("White space is not allowed for the 'account' attribute.");
     }
 
     $logger->debug("Returning $rv.");
@@ -327,7 +329,6 @@ sub _validate {
 }
 
 
-
 # This method knows how to dispatch method invocations to the proper module
 # or class by checking the name against the hashes set up early in this
 # module. The hashes are used to look up which methods go where.
@@ -362,14 +363,16 @@ sub AUTOLOAD {
 # method. Don't modify or remove unless you know what you are doing.
 # This is a private method that is not to be invoked directly.
 sub DESTROY { 
-    # Close the DRMAA Session
-    $logger->debug("Closing the DRMAA session.");
-    my ($error, $diagnosis) = drmaa_exit();
-    if ($error) {
-        $logger->error("Error closing the DRMAA session: ",
-                 drmaa_strerror($error), $diagnosis);
-    } else {
-        $DRMAA_INITIALIZED = 0;
+    # Close the DRMAA Session (if necessary).
+    if ( $DRMAA_INITIALIZED != 0 ) {
+        $logger->debug("Closing the DRMAA session.");
+        my ($error, $diagnosis) = drmaa_exit();
+        if ($error) {
+            $logger->error("Error closing the DRMAA session: ",
+                     drmaa_strerror($error), $diagnosis);
+        } else {
+            $DRMAA_INITIALIZED = 0;
+        }
     }
 }
 
@@ -395,35 +398,38 @@ order in which parameters are added. They are reassembled at runtime on the grid
 the same order that they were added...
 
 B<Parameters:> If the number of arguments is 1, then it will be considered to
-be a simple, "anonymous" parameter... 
-When called with a single scalar argument, no logic is
-attempted to interpret the string provided. The module simply adds the
-specified parameter verbatim to the list of parameters when building the
-command line to invoke on the grid. 
-If 3 parameters are passed, then they are read as "key", "value", "type". The
-type can be either "ARRAY", "DIR", "PARAM", "FILE", or "TEMPFILE"
-(the default is "PARAM" when less than 3 arguments are passed). The type is
-used in the following way to aid in the parallelization of processes: If ARRAY
-is used, the job will be iterated over the elements of the array, with the
-value of the parameter being changed to the next element of the array each
-time. The array must be an array of simple strings passed in as an array
-reference to VALUE. Newlines will be stripped. Note: Nested data structures
-will not be respected. If DIR is specified, the file contents of the directory
-will be iterated over. If a directory contains 25 files, then there will be at
-least 25 jobs, with the name of each file being a parameter value for each
-invocation. If FILE is specified, then the VALUE specified in the method call
+be a simple, "anonymous" parameter and ...  When called with a single scalar
+argument, no logic is attempted to interpret the string provided. The module
+simply adds the specified string verbatim to the list of parameters when
+building the command line to invoke on the grid.  If 3 parameters are passed,
+then they are read as "key", "value", "type". The parameter 'type' can be
+either "ARRAY", "DIR", "PARAM", or "FILE" (the default is "PARAM"). Any other
+number of parameters passed will yield an error.
+
+The 'type' is used in the following way to aid in the parallelization of
+processes: If ARRAY is used, the job will be iterated over the elements of the
+array, with the value of the parameter being changed to the next element of the
+array each time. The array must be an array of simple strings passed in as an
+array reference to 'value'. Newlines will be stripped.  Note: Nested data
+structures will not be respected.
+
+If "DIR" is specified as the 'type', the file contents of the directory
+specified by the 'value' will be iterated over.  If the directory contains 25
+files, then there will be at least 25 invocations of the executable on the grid
+( one per file) with the name of each file substituted for the '$(Name)' token
+each time. Note that hidden files and directories are not counted and the
+directory is NOT scanned recursively.
+
+If "FILE" is specified, then the 'value' specified in the method call
 will be interpreted as the path to a file containing entries to iterate over.
 The file may contain hundreds of entries (1 per line) to generate a
-corresponding number of jobs. TEMPFILE works similarly to FILE, except that the
-HTC system will delete, or clean up the file when the request has finished
-being processed. Use with caustion. Finally, PARAM, the default type,
-provides simple parameter support and no iteration will occur.
+corresponding number of jobs.
 
 If greater clarity and flexibility is desired, one may wish to pass named
 parameters in a hash reference instead:
 
-  $obj->add_param( { key   => "--someparam",
-                     value => "somevalue",
+  $obj->add_param( { key   => '--someparam=$(Name)',
+                     value => "/path/to/directory",
                      type  => "DIR",
                    });
 
@@ -454,12 +460,12 @@ Examples:
 
    DIR
       $request->add_param({ type   => "DIR",
-                            key    => "--filepath=$(Name)',
+                            key    => '--filepath=$(Name)',
                             value  => "/path/to/some/directory",
                          });
    ARRAY
       $request->add_param({ type   => "ARRAY",
-                            key    => "--element=$(Name)',
+                            key    => '--element=$(Name)',
                             value  => \@array,
                          });
 
@@ -501,6 +507,48 @@ sub add_param {
     return $return;
 }
 
+=item $obj->block_size( [ $scalar | $code_ref ] );
+
+B<Description:> By default, Master/Worker (mw) jobs have a default block size
+of 100.  That is to say, that each worker on the grid will process 100 elements
+of the overall pool of job invocations. However, this isn't always appropriate.
+The user may override the default block size by calling this method and setting
+the block size to an alternate value (a positive integer). The user may also
+provide an anonoymous subroutine (code reference) so that the block size can be
+computed dynamically. If choosing to pass a subroutine , the code reference
+will be passed two arguments: the Grid::Request::Command object that will be
+invoked, and the number of elements that will be iterated over, in that order.
+The subroutine can then use these pieces of information to compute the block
+size. The subroutine MUST return a positive integer scalar or an exception will
+be thrown.
+
+    Examples:
+        # simple scalar block size
+        $request->block_size(1000);
+
+        # Passing a code ref, to make the block size dependent on the
+        # executable...
+        $request->block_size(
+                      sub {
+                          my $com_obj = shift;
+                          my $count = shift;
+
+                          my $exe = $com_obj->command();
+
+                          my $block_size = 50;
+                          if ($exe =~ m/sort/i) {
+                              $block_size = ($count > 100000) ? 10000 : 1000;
+                          }
+                          return $block_size;
+                      }
+                  );
+
+B<Parameters:> A positive integer scalar, or an anonymous subroutine/code
+reference.
+
+B<Returns:> The block size scalar or code reference if called as an accessor
+(no-arguments). If the block size has not been explicitly set, then the default
+block size is returned. No return if called as a mutator.
 
 =item $obj->class([$class]);
 
@@ -525,12 +573,13 @@ will be called for the request.
 B<Parameters:> With no parameters, this method functions as a getter. With one
 parameter, the method sets the executable. Currently, this module does not
 attempt to verify whether the exeutable is actually present or whether
-permissions on the executable allow it to be called by the DCE.
+permissions on the executable allow it to be invoked by the user on grid
+machines.
 
 B<Returns:> The currently set executable (when called with no arguments).
 
 
-=item $obj->email([$command]);
+=item $obj->email([$email_address]);
 
 B<Description:> This method is used to set or retrieve the email of the user
 submitting the request. The email is important for notifications and for
@@ -551,9 +600,9 @@ B<Description:> Retrieve the finish time of the request.
 
 B<Parameters:> None.
 
-B<Returns:> The ending time of the request (the time the DCE finished
-processing the request), or undef if the end_time has not yet been
-established.
+B<Returns:> The ending time of the request (the time the grid finished
+processing the request), or undef if the ending time has not yet been
+determined.
 
 
 =item $obj->error([errorfile])
@@ -561,12 +610,19 @@ established.
 B<Description:> This method allows the user to set, or if the request has not
 yet been submitted, to reset the error file. The error file will be the place
 where all STDERR from the invocation of the executable will be written to. This
-file should be in a globally accessible location on the filesystem. The
-attribute may not be changed with this method once the request has been
-submitted.
+file should be in a globally accessible location on the filesystem such that
+grid execution machines may create the files. The attribute may not be changed
+with this method once the request has been submitted.
 
 B<Parameters:> To set the error file, call this method with one parameter,
-which should be the path to the file where STDERR is to be written.
+which should be the path to the file where STDERR is to be written.  Note that
+when submitting array jobs (with the use of the times() method or with
+Master/Worker parameters through add_param()), one can also use the $(Index)
+token when specifying the error path. The token will be replaced with the
+grid's task ID number. For example, if a request generated 100 grid jobs, then
+an error path containing '/path/to/directory/job_$(Index).err' will result in
+STDERR files numbered job_1.err, job_2.err, ..., job_100.err in
+/path/to/directory.
 
 B<Returns:> When called with no arguments, this method returns the currently
 set error file, or undef if not yet set.
@@ -575,11 +631,11 @@ set error file, or undef if not yet set.
 =item $obj->getenv([1]);
 
 B<Description:> The getenv method is used to set whether the user's environment
-should be replicated to the DCE or not. To replicate your environment, call
+should be replicated to the grid or not. To replicate your environment, call
 this method with an argument that evaluates to true. Calling it with a 0
 argument, or an expression that evaluates to false, will turn off environment
 replication. The default is NOT to replicate the user environment across the
-DCE.
+grid.
 
 B<Parameters:> This method behaves as a getter when called with no arguments.
 If called with 1, or more arguments, the first will be used to set the
@@ -643,57 +699,47 @@ sub is_submitted {
 }
 =item $obj->project([$project]);
 
-B<Description:> The project attribute is used to affiliate usage of the DRM with
-a particular administrative project. This will allow for more effective
+B<Description:> The project attribute is used to affiliate usage of the DRM
+with a particular administrative project. This will allow for more effective
 control and allocation of resources, especially when high priority projects
-must be fulfilled. Therefore, the "project" is mandatory when the request object
-is built. However, the user may still change the project attribute as long as
-the job has not yet been submitted (after submission most attributes are
+must be fulfilled. The caller may change the project setting as long as the job
+has not yet been submitted (after submission most request attributes are
 locked).
 
 B<Parameters:> The first parameter will be used to set (or reset)
 the project attribute for the request, as long as the request has not
 been submitted.
 
-B<Returns:> The currently set project (if called with no parameters).
+B<Returns:> The currently set project (if called with no arguments).
 
 
 =item $obj->input([path]);
 
-B<Description:>
+B<Description:> Used to specify a file to be used as the STDIN for
+the executable on the grid.
 
-B<Parameters:>
+B<Parameters:> A scalar containing the globally accessible path to
+the file to use for STDIN.
 
-B<Returns:>
+B<Returns:> The currently set input file if called as a getter with no
+arguments, or undef if not yet set.
 
 
 =item $obj->initialdir([path]);
 
-B<Description:> This method sets the directory where the DCE will be
+B<Description:> This method sets the directory where the grid will be
 chdir'd to before invoking the executable. This is an optional parameter,
-and if the user leaves it unspecified, the default will be that the DCE
-will be chdir'd to the root directory "/" before beginning the request.
-Use of initialdir is encouraged to promote use of relative paths.
+and if the user leaves it unspecified, the default will be that the grid
+job will be chdir'd to the root directory "/" before beginning the request.
+Use of initialdir is encouraged to promote the use of relative paths.
 
-B<Parameters:> A scalar holding the path to the directory the DCE should
+B<Parameters:> A scalar holding the path to the directory the grid should
 chdir to before invoking the executable.
 
 B<Returns:> When called with no arguments, returns the currently set
 initialdir, or undef if not yet set.
 
 
-=item $obj->length([length]);
-
-B<Description:> This method is used to characterize how long the request
-is expected to take to complete. For long running requests, an attempt to
-match appropriate resources is made. If unsure, leave this setting unset.
-
-B<Parameters:> "short", "medium", "long". No attempt is made to validate
-the length passed in when used as a setter.
-
-B<Returns:> The currently set length attribute (when called with no
-arguments).
-
 =item $obj->name([name]);
 
 B<Description:> The name attribute for request objects is optional and is
@@ -705,19 +751,6 @@ B<Returns:> When called with no arguments, returns the current name, or
 undef if not yet set. The name cannot be changed once a request is submitted.
 
 
-=item $obj->next_command();
-
-B<Description:> The module allows for requests to encapsulate multiple
-commands. This method will start work on a a new one by moving a cursor.
-Commands are processed in the order in which they are created. In addition,
-the only attribute that the new command inherits from the command that
-preceded it, is the project. However, users are free to change the project by
-calling the project() method...
-
-B<Parameters:> None.
-
-B<Returns:> None.
-
 =item $obj->opsys([$os]);
 
 B<Description:> The default operating system that the request will be processed
@@ -755,10 +788,52 @@ B<Returns:> When called with no arguments, returns the hosts if set.
 
 B<Description:> Used to set the minimum amount of physical memory needed.
 
-B<Parameters:> memory in megabytes, example 10MB, 512MB
+B<Parameters:> memory in megabytes. Examples: 1000MB, 5000MB
 
 B<Returns:> When called with no arguments, returns the memory if set.
 
+=item $obj->new_command();
+
+B<Description:> The module allows for requests to encapsulate multiple
+commands. This method will start work on a a new one by moving a cursor.
+Commands are processed in the order in which they are created if they are
+submitted synchronously, or in parallel if submitted asynchronously (the
+default). In addition, the only attribute that the new command inherits from
+the command that preceded it, is the project (if set). However, users are free
+to change the project by calling the project() method...
+
+B<Parameters:> None.
+
+B<Returns:> None.
+
+=cut
+
+sub new_command {
+    $logger->debug("In new_command.");
+    
+    my ($self, @args) = @_;
+    if (scalar @args) {
+        Grid::Request::InvalidArgumentException->throw("No arguments are valid for new_command().");
+    }
+
+    # The only piece of information replicated from command to command is the
+    # project. So we first get the project and then use it to build the new
+    # Command object.
+    my $project = $self->project();
+
+    # Increment element pointer.
+    my @objs = $self->_com_obj_list;
+    my $cmd_len = scalar(@objs);
+
+    $logger->debug("Creating new Command object in element $cmd_len.");
+    if (defined $project && length($project)) {
+        $self->_com_obj_list->[$cmd_len] =
+            Grid::Request::Command->new( project => $project );
+    } else {
+        $self->_com_obj_list->[$cmd_len] = Grid::Request::Command->new();
+    }
+}
+
 
 =item $obj->pass_through([pass_value]);
 
@@ -768,38 +843,27 @@ request's requirements. Such pass throughs are forwarded unchanged. This is an
 advanced option and should only be used by those familiar with the the
 underlying DRM.
 
-B<Parameters:> $string
+B<Parameters:> $string, a scalar.
 
 B<Returns:> None.
 
-=cut
-
-sub new_command {
-    $logger->debug("In new_command.");
-    my $self = shift;
-
-    # The only piece of information replicated from command to command is the
-    # project. So we first get the project and then use it to build the new
-    # Command object.
-    my $project = $self->project();
-
-    # Increment element pointer.
-    $command_element++;
-    $logger->debug("Creating Command object in element $command_element.");
-    $self->_com_obj_list->[$command_element] =
-        Grid::Request::Command->new( project => $project );
-}
 
 =item $obj->output([path]);
 
 B<Description:> Sets the path for the output file, which would hold all of
-the output directed to STDOUT by the request on the DCE. This method functions
+the output directed to STDOUT by the request on the grid. This method functions
 as a setter and getter.
 
-B<Parameters:> A path to a file. The file must be globally accessible on
-the filesystem in order to work, otherwise, the location will not be accessible
-to compute nodes on the DCE. This attribute may not be changed once a request
-is submitted.
+B<Parameters:> A path to a file. The file must be globally accessible on the
+filesystem in order to work, otherwise, the location will not be accessible to
+compute nodes on the grid. This attribute may not be changed once a request is
+submitted. Note that when submitting array jobs (with the use of the times()
+method or with Master/Worker parameters through add_param()), one can also use
+the $(Index) token when specifying the output path. The token will be replaced
+with the grid's task ID number. For example, if a request generated 100 grid
+jobs, then an output path containing '/path/to/directory/job_$(Index).out' will
+result in STDOUT files numbered job_1.out, job_2.out, ..., job_100.out in
+/path/to/directory.
 
 B<Returns:> When called with no arguments, the method returns the currently
 set path for the output file, or undef if not yet set.
@@ -826,16 +890,25 @@ B<Parameters:> Scalar priority value.
 
 B<Returns:> The current priority, or undef if unset.
 
-=cut
+=item $obj->runtime([minutes]);
+
+B<Description:> Cap the runtime of the job on the DRM. Most DRM systems
+have a mechanism to limit the maximum amount of time a job can run. This
+method accepts a value in minutes.
+
+B<Parameters:> Scalar containing a positive integer nubmer of minutes.
+
+B<Returns:> The current runtime limit if called as a getter, or undef if unset.
+
 
 =item $obj->set_env_list(@vars);
 
-B<Description:> This method is used to establish the environment that a a
+B<Description:> This method is used to establish the environment that a
 request to the grid should run under. Users may pass this method a list of
 strings that are in "key=value" format. The keys will be converted into
 environment variables set to "value" before execution of the command is begun.
 Normally, a request will not copy the user's environment in this way.  The only
-time the environment is established on the DCE will be if the user invokes the
+time the environment is established on the grid will be if the user invokes the
 getenv method or sets it with this method. This  method allows the user to
 override the environment with his or her own notion of what the environment
 should be at runtime on the grid.
@@ -858,7 +931,7 @@ sub set_env_list {
         push(@valid, $arg);
     }
 
-    $self->[5] = \@valid;
+    $self->{_env} = \@valid;
 
     # If the user has set their own environment with set_envlist, then we
     # assume that they want getenv to be true. We do it for them here to save
@@ -867,6 +940,63 @@ sub set_env_list {
 }
 
 
+=item $obj->show_invocations();
+
+B<Description:> Show what will be executed on the DRM. An attempt is made
+to excape shell sensitive characters so that the commands can be copied
+and pasted for test execution.
+
+B<Parameters:> None.
+
+B<Returns:> None. The method print to STDOUT.
+
+=cut
+
+sub show_invocations {
+    my $self = shift;
+
+    # Remember, we may have multiple command objects...
+    my @command_objs = $self->_com_obj_list();
+
+    # Iterate over them...
+    my $command_count = 1;
+
+    eval { require Grid::Request::JobFormulator };
+    my $formulator = Grid::Request::JobFormulator->new();
+
+    foreach my $com_obj (@command_objs) {
+        print "Command #" . $command_count . "\n"; 
+        my $exe = $com_obj->command();
+        my $block_size = $com_obj->block_size();
+        my @params = $com_obj->params();
+        my @param_strings = ();
+
+        foreach my $param_obj (@params) {
+            my $param_str = $param_obj->to_string();
+            push (@param_strings, $param_str);
+        }
+
+        my @invocations = $formulator->formulate($block_size, $exe, @param_strings);
+        foreach my $invocations (@invocations) {
+            my @cli = @$invocations;
+            my @esc_cli = _esc_chars(@cli);
+            print join(" ", @esc_cli) . "\n";
+        }
+        
+        $command_count++;
+    }
+}
+
+sub _esc_chars {
+    # will change, for example, a!!a to a\!\!a
+    my @cli = @_;
+    my $e;
+    @cli = map { $e =$_; $e =~ s/([;<>\*\|`&\$!#\(\)\[\]\{\}:'"])/\\$1/g; $e } @cli;
+    @cli = map { if ($_ =~ m/\s/) { '"' . $_ . '"' } else { $_ } } @cli;
+    return @cli;
+}
+
+
 =item $obj->simulate([value]);
 
 B<Description:> This method is used to toggle the simulate flag for the
@@ -1038,7 +1168,9 @@ sub _throw_drmaa {
 # implementation.
 sub _init_drmaa {
     $logger->debug("In _init_drmaa.");
-    unless ($DRMAA_INITIALIZED) {
+    if ($DRMAA_INITIALIZED) {
+        $logger->debug("DRMAA session aready initialized.");
+    } else {
         my ($error, $diagnosis) = drmaa_init("session=$SESSION_NAME");
         _throw_drmaa("Could not initialize DRMAA", $error, $diagnosis) if $error;
         $DRMAA_INITIALIZED = 1;
@@ -1074,23 +1206,41 @@ sub _drmaa_submit {
     my @ids;
     my $total = $self->command_count();
     my $count = 1;
-    for (my $cmd = 0; $cmd < $total; $cmd++) {
-        my $cmd_obj = $self->_com_obj_list->[$cmd];
-        $logger->debug("Submitting command $count/$total.");
-        my $jt = $self->_cmd_base_drmaa($cmd_obj);
-        $logger->debug("Got a good job template.") if defined $jt;
-        my @sub_ids;
-        if ($cmd_obj->cmd_type() eq "mw") {
-            @sub_ids = $self->_submit_mw($jt, $cmd_obj);
-        } else {
-            @sub_ids = $self->_submit_htc($jt, $cmd_obj);
+
+    eval {
+        for (my $cmd = 0; $cmd < $total; $cmd++) {
+            my $cmd_obj = $self->_com_obj_list->[$cmd];
+            $logger->debug("Submitting command $count/$total.");
+            my $jt = $self->_cmd_base_drmaa($cmd_obj);
+            $logger->debug("Got a good job template.") if defined $jt;
+            my @sub_ids;
+            if ($cmd_obj->cmd_type() eq "mw") {
+                @sub_ids = $self->_submit_mw($jt, $cmd_obj);
+            } else {
+                @sub_ids = $self->_submit_htc($jt, $cmd_obj);
+            }
+            if ($serially) {
+                _sync_ids($cmd_obj);
+            }
+            push @ids, @sub_ids;
+            $count++;
         }
-        if ($serially) {
-            _sync_ids($cmd_obj);
+    };
+
+    my $e;
+    if ($e = Exception::Class->caught("Grid::Request::DRMAAException")) {
+        $logger->fatal("Unable to submit: " . $e->diagnosis() . ": " . $e->drmaa() );
+        $e->rethrow();
+    } elsif ( $e = Exception::Class->caught("Grid::Request::Exception") ) {
+        $logger->fatal("Unable to submit: " . $e->error());
+        $e->rethrow();
+    } else {
+        $e = Exception::Class->caught();
+        if ($e) {
+            ref $e ? $e->rethrow : $logger->logcroak("Unable to submit: $e");
         }
-        push @ids, @sub_ids;
-        $count++;
     }
+
     $logger->debug("Finished submitting.");
     return wantarray ? @ids : \@ids;
 }
@@ -1117,8 +1267,8 @@ sub _cmd_base_drmaa {
     my $output = $cmd->output();
     my $error_path = $cmd->error();
     if (defined($output) || defined($error_path)) {
-        $output =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g;
-        $error_path =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g;
+        $output =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g if defined($output);
+        $error_path =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g if defined($error_path);
 
         $output ||= '/dev/null';
         $logger->debug("STDOUT will go to $output.");
@@ -1161,7 +1311,7 @@ sub _cmd_base_drmaa {
     my $email = $cmd->email();
     if ($email) {
         $logger->info("Setting DRM to not block emails.");
-        ($error, $diagnosis) = drmaa_set_attribute($jt, $DRMAA_BLOCK_EMAIL, 0);
+        ($error, $diagnosis) = drmaa_set_attribute($jt, $DRMAA_BLOCK_EMAIL, "0");
         _throw_drmaa("Unable to unblock emails.", $error, $diagnosis) if $error;
         $logger->info("Setting the job email.");
         ($error, $diagnosis) = drmaa_set_vector_attribute($jt, $DRMAA_V_EMAIL, [$email]);
@@ -1169,7 +1319,7 @@ sub _cmd_base_drmaa {
     }
  
     my @drm_methods = qw(account hosts opsys evictable priority memory
-                         length project class runtime);
+                         project class runtime);
     my @native_attrs;
     foreach my $method (@drm_methods) {
         my $val = $cmd->$method;
@@ -1208,7 +1358,7 @@ sub _submit_mw {
     $logger->debug("In _submit_mw.");
     my ($self, $jt, $cmd) = @_;
     unless (defined $jt && defined $cmd) {
-        Grid::Request::InvalidArgumentException->("Job template and/or command object are not defined.");
+        Grid::Request::InvalidArgumentException->throw("Job template and/or command object are not defined.");
     }
 
     $logger->debug("Setting the command executable.");
@@ -1263,23 +1413,31 @@ sub _submit_mw {
     # should be split up differently. This is why we pass $cmd, so that more
     # intelligent analysis may be done if configured...
     
-    my $block_size = $DEFAULT_BLOCK_SIZE;
+    my $block_size = $cmd->block_size();
+    if (ref($block_size) eq "CODE") {
+        $logger->debug("Detected a code reference for block size.");
+        my $block_size_calculator = $block_size;
+        $logger->debug("Invoking the code to determine the block size.");
+        $block_size = $block_size_calculator->($cmd, $min_count); 
+
+        if (defined($block_size) && length($block_size) && $block_size =~ /^-?\d+$/) {                                                                          
+            if ($block_size > 0) {                                                                              
+                $logger->debug("Invocation yielded a block size of $block_size.");
+            } else {
+                Grid::Request::Exception->throw(                                                  
+                    "Block size code reference yielded an invalid result. Must be a positive integer.");
+            }
+        } else {
+            Grid::Request::Exception->throw(
+                    "Block size code reference yielded an invalid result.");
+        }
+    } else {
+        $logger->debug("block_size is a regular scalar: $block_size");
+    }
+
+    # Compute the number of workers to invoke based on the block size.
     my $workers = ceil($min_count / $block_size);
 
-    eval { 
-        my $package = "use Grid::Request::Util";
-        my $load_result = eval "require $package";
-        if (defined($load_result) && ($load_result == 1)) {
-            import $package;
-            # Use some (possibly user defined) technique to deterine the block
-            # size based on what kind of job is being executed on the grid
-            $block_size = Grid::Request::Util->determine_block_size($cmd, $min_count);
-            $workers = ceil($min_count / $block_size);
-        }
-    };
-    if ($@) {
-        $logger->error("Could not load the Grid::Request::Util module. Sticking with the worker count of $workers.");
-    }
     my $plurality = ($workers == 1) ? "worker" : "workers";
     $logger->info("This master/worker command requires $workers $plurality.");
 
@@ -1348,13 +1506,13 @@ sub _submit_htc {
     $logger->debug("In _submit_htc.");
     my ($self, $jt, $cmd) = @_;
     unless (defined $jt && defined $cmd) {
-        Grid::Request::InvalidArgumentException->(
+        Grid::Request::InvalidArgumentException->throw(
             "Job template and/or command object are not defined.");
     }
 
     my $exe = $cmd->command();
     unless (defined $exe) {
-        Grid::Request::InvalidArgumentException->("Command executable is not defined.");
+        Grid::Request::InvalidArgumentException->throw("Command executable is not defined.");
     }
     $logger->debug("Setting the command executable.");
     my ($error, $diagnosis) = drmaa_set_attribute($jt, $DRMAA_REMOTE_COMMAND, $exe);
