@@ -8,6 +8,8 @@ Grid::Request::Test - Helper funcations for unit Grid::Request tests.
 
  use Grid::Request::Test;
 
+ my $req = Grid::Request:Test->get_test_request();
+
  my $project = Grid::Request:Test->get_test_project();
 
  my $hostname = Grid::Request:Test->get_test_host();
@@ -18,6 +20,42 @@ A module that models Grid::Request parameters.
 
 =over 4
 
+=item get_test_request();
+
+B<Description:> This method is used by tests to obtain a test request
+object. If the user has the GRID_REQUEST_TEST_PROJECT enviroment variable
+defined, for environments in which a project settings is required, then
+the returned object will have the project configured so that tests will
+run.
+
+B<Parameters:> None.
+
+B<Returns:> A Grid::Request object.
+
+=cut 
+
+use Carp;
+use Grid::Request;
+use Test::More;
+
+our $GR_PROJECT_NAME = "GRID_REQUEST_TEST_PROJECT";
+our $GR_HOST_NAME = "GRID_REQUEST_TEST_HOST";
+
+our $VERSION = '0.11';
+
+
+sub get_test_request {
+    my $req;
+    my $project = get_test_project();
+
+    if (defined $project && length $project) {
+        $req = Grid::Request->new(project => $project);
+    } else {
+        $req = Grid::Request->new();
+    }
+    return $req;
+}
+
 =item get_test_project();
 
 B<Description:> Retrieves the project to use when running the
@@ -31,22 +69,12 @@ B<Returns:> A scalar.
 
 =cut 
 
-my $GR_PROJECT_NAME = "GRID_REQUEST_TEST_PROJECT";
-my $GR_HOST_NAME = "GRID_REQUEST_TEST_HOST";
-
-our $VERSION = '0.10';
-
 sub get_test_project {
-    if (exists $ENV{$GR_PROJECT_NAME} &&
-        defined $ENV{$GR_PROJECT_NAME} &&
-        length($ENV{$GR_PROJECT_NAME})) {
+    if (exists $ENV{$GR_PROJECT_NAME} && defined $ENV{$GR_PROJECT_NAME} &&
+            length($ENV{$GR_PROJECT_NAME})) {
 
         # Return the value.
-        # TODO: Validation.
         return $ENV{$GR_PROJECT_NAME};
-    } else {
-        warn "Please define the \"$GR_PROJECT_NAME\" environment variable.\n";
-        exit 1;
     }
 }
 
@@ -72,7 +100,19 @@ sub get_test_host {
         return $ENV{$GR_HOST_NAME};
     } else {
         warn "Please define the \"$GR_HOST_NAME\" environment variable.\n";
-        exit 1;
+        croak;
+    }
+}
+
+sub diagnose {
+    my $e;
+    if ( $e = Exception::Class->caught('Grid::Request::DRMAAException') ) {
+        diag($e->diagnosis());
+    } elsif ( $e = Exception::Class->caught('Grid::Request::Exception') ) {
+        diag($e->error());
+    } else {
+        $e = Exception::Class->caught();
+        diag($e);
     }
 }
 
@@ -98,3 +138,4 @@ None known.
 =head1 SEE ALSO
 
  Grid::Request
+ Test::More
